﻿using Mapbox.Unity.Location;
using Mapbox.Utils;
using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class DataPath : MonoBehaviour
{

  public class DataPoint
  {
    public string id;
    public float speed;
    public double lat;
    public double lon;
    public int direction;
    public long timestamp;
  }

  public string id = "";
  public List<DataPoint> points = new List<DataPoint>();
  private Vector3[] positions;
  private Color32 color;

  public static Color32 ConvertAndroidColor(int aCol)
  {
    Color32 c = new Color32();
    c.b = (byte)((aCol) & 0xFF);
    c.g = (byte)((aCol >> 8) & 0xFF);
    c.r = (byte)((aCol >> 16) & 0xFF);
    c.a = 0xFF;
    return c;
  }

  public void Setup()
  {
    int hash = this.id.GetHashCode();
    this.color = ConvertAndroidColor(hash);

    //points.Sort(System.Comparison<DataPoint>.cre)
    points.Sort(Comparer<DataPath.DataPoint>.Create((l,r) => isign(l.timestamp - r.timestamp)));


    CalcWorldLines();
    LocationProviderFactory.Instance.mapManager.OnUpdated += () =>
    {
      CalcWorldLines();
    };
  }

  private int isign(long s)
  {
    if(s < 0) return -1;
    if(s > 0) return 1;
    return 0;
  }

  private void CalcWorldLines()
  {
    var lr = this.GetComponent<LineRenderer>();
    Vector3[] positions = new Vector3[points.Count];
    Color[] colors = new Color[points.Count];
    int hash = this.id.GetHashCode();
    Color32 col = ConvertAndroidColor(hash);

    var map = LocationProviderFactory.Instance.mapManager;
    //Vector3 prevPosition = map.GeoToWorldPosition(new Vector2d(points[0].lat, points[0].lon));
    for (int i = 0; i < points.Count; ++i)
    {
      Vector3 position = map.GeoToWorldPosition(new Vector2d(points[i].lat, points[i].lon));
      //DEBUG
      //position.y = 1;
      positions[i] = position;
      colors[i] = col;

      //Debug.DrawLine(prevPosition, position, col);
      //prevPosition = position;
    }

    //lr.SetColors()
    this.positions = positions;
    lr.positionCount = positions.Length;
    lr.SetPositions(positions);
    lr.startColor = col;
    lr.endColor = col;


    Gradient gradient;
    GradientColorKey[] colorKey;
    GradientAlphaKey[] alphaKey;

    gradient = new Gradient();

    // Populate the color keys at the relative time 0 and 1 (0 and 100%)
    colorKey = new GradientColorKey[2];
    

    colorKey[0].color = Color.Lerp(col, Color.black, 0.5f);
    colorKey[0].time = 0.0f;
    colorKey[1].color = Color.Lerp(col, Color.white, 0.5f);
    colorKey[1].time = 1.0f;

    // Populate the alpha  keys at relative time 0 and 1  (0 and 100%)
    alphaKey = new GradientAlphaKey[2];
    alphaKey[0].alpha = 1.0f;
    alphaKey[0].time = 0.0f;
    alphaKey[1].alpha = 0.0f;
    alphaKey[1].time = 1.0f;

    gradient.SetKeys(colorKey, alphaKey);

    lr.colorGradient = gradient;


    //DEBUG
    //lr.enabled = false;

    this.transform.position = positions[0];
  }

  void LateUpdate()
  {
    if(positions == null || positions.Length == 0) return;
    

    Vector3 prevPosition = positions[0];
    for (int i = 1; i < positions.Length; ++i)
    {
      Vector3 position = positions[i];
      //Debug.DrawLine(prevPosition + Vector3.up, position + Vector3.up, color, 0, true);;
      prevPosition = position;
    }

  }

  private void OnDrawGizmos()
  {
    if(positions == null || positions.Length == 0) return;

    int i = ((int)Time.time * 2) % positions.Length; 

    Gizmos.color = this.color;
    Gizmos.DrawSphere(positions[i], 1);
  }

}
